/*
  JWildfire - an image and animation processor written in Java 
  Copyright (C) 1995-2011 Andreas Maschke
  This is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser 
  General Public License as published by the Free Software Foundation; either version 2.1 of the 
  License, or (at your option) any later version.
 
  This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
  Lesser General Public License for more details.
  You should have received a copy of the GNU Lesser General Public License along with this software; 
  if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jwildfire.create.tina.variation;

import org.jwildfire.create.tina.base.XForm;
import org.jwildfire.create.tina.base.XYZPoint;
import static org.jwildfire.base.mathlib.Complex;
import static org.jwildfire.base.mathlib.MathLib.*;


public class Template_Func extends VariationFunc {
  private static final long serialVersionUID = 1L;
  
  private static final String PARAM_ZX = "zx";
  private static final String PARAM_ZY = "zy";
    
  private static final String[] paramNames = { PARAM_ZX, PARAM_ZY};
  private double zx = 1.0;
  private double zy = 0.0;

    
  @Override
  public void transform(FlameTransformationContext pContext, XForm pXForm, XYZPoint pAffineTP, XYZPoint pVarTP, double pAmount) {
  // by Whittaker Courtney 

  

       Complex z = new Complex(pAffineTP.x, pAffineTP.y);
       Complex z2 = new Complex(pAffineTP.x, pAffineTP.y);
       Complex z6 = new Complex(zx, zy);


z.Exp();
z.Add(z6);

z2.Exp();
z2.Sub(z6);

z.Div(z2);
z.Sqrt();
z.AcosecH();
       z.Scale(pAmount * M_2_PI);
                                                                   
   if (pContext.random() < 0.5){                              
       pVarTP.x += z.re;  
       pVarTP.y += z.im;  
    }
    else{     
       pVarTP.x += -z.re; 
       pVarTP.y += -z.im;
    }                   
             
	if (pContext.isPreserveZCoordinate()) {
		pVarTP.z += pAmount * pAffineTP.z;
	}

  }
  
  @Override
  public String[] getParameterNames() {
    return paramNames;
  }

  @Override
  public Object[] getParameterValues() {
    return new Object[] {zx, zy};
  }

  @Override
  public void setParameter(String pName, double pValue) {
    if (PARAM_ZX.equalsIgnoreCase(pName))
      zx = pValue;
    else if (PARAM_ZY.equalsIgnoreCase(pName))
      zy = pValue;         
    else
      throw new IllegalArgumentException(pName);
  }
 

  @Override
  public String getName() {
    return "template";
  }

}
